@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2008 Massachusetts Institute of Technology
+/* Copyright (C) 2005-2009 Massachusetts Institute of Technology
 %
 %  This program is free software; you can redistribute it and/or modify
 %  it under the terms of the GNU General Public License as published by
@@ -37,8 +37,8 @@ double fields::count_volume(component c) {
 
 double fields_chunk::count_volume(component c) {
   double vol = 0;
-  for (int i=0;i<v.ntot();i++)
-    vol += v.dV(c,i).full_volume();
+  for (int i=0;i<gv.ntot();i++)
+    vol += gv.dV(c,i).full_volume();
   return vol;
 }
 
@@ -50,11 +50,11 @@ double fields::field_energy() {
   return field_energy_in_box(user_volume.surroundings());
 }
 
-double fields::energy_in_box(const geometric_volume &where) {
+double fields::energy_in_box(const volume &where) {
   return thermo_energy_in_box(where) + field_energy_in_box(where);
 }
 
-double fields::field_energy_in_box(const geometric_volume &where) {
+double fields::field_energy_in_box(const volume &where) {
   synchronize_magnetic_fields();
   double cur_step_magnetic_energy = magnetic_energy_in_box(where);
   restore_magnetic_fields();
@@ -69,8 +69,8 @@ static complex<double> dot_integrand(const complex<double> *fields,
 }
 
 double fields::field_energy_in_box(component c,
-				   const geometric_volume &where) {
-  if (coordinate_mismatch(v.dim, c))
+				   const volume &where) {
+  if (coordinate_mismatch(gv.dim, c))
     return 0.0;
 
   component cs[2];
@@ -88,14 +88,14 @@ double fields::field_energy_in_box(component c,
   return real(integrate(2, cs, dot_integrand, 0, where)) * 0.5;
 }
 
-double fields::electric_energy_in_box(const geometric_volume &where) {
+double fields::electric_energy_in_box(const volume &where) {
   long double sum = 0.0;
   FOR_ELECTRIC_COMPONENTS(c)
     sum += field_energy_in_box(c, where);
   return sum;
 }
 
-double fields::magnetic_energy_in_box(const geometric_volume &where) {
+double fields::magnetic_energy_in_box(const volume &where) {
   long double sum = 0.0;
   FOR_MAGNETIC_COMPONENTS(c)
     sum += field_energy_in_box(c, where);
@@ -104,39 +104,47 @@ double fields::magnetic_energy_in_box(const geometric_volume &where) {
 
 void fields_chunk::backup_component(component c) {
   DOCMP {
-    if (f[c][cmp] && 
+    if (c < NUM_FIELD_COMPONENTS && f[c][cmp] && 
 	// in mu=1 regions where H==B, don't bother to backup H
-	!(is_magnetic(c) && f[c][cmp] == f[Bx + (c - Hx)][cmp])) {
+	!(is_magnetic(c) && f[c][cmp] 
+	  == f[direction_component(Bx, component_direction(c))][cmp])) {
 
-      // if we have f_minus_p then f_prev stores this, and we need
-      // to implement an extra backup field ... punt for now
-      if (f_minus_p[c][cmp]) abort("backup with fields-P not implemented");
+#define BACKUP(f) if (f[c][cmp]) {					\
+      if (!f##_backup[c][cmp])						\
+	f##_backup[c][cmp] = new realnum[gv.ntot()];			\
+      memcpy(f##_backup[c][cmp], f[c][cmp], gv.ntot()*sizeof(realnum)); }
 
-      if (!f_backup[c][cmp])
-	f_backup[c][cmp] = new double[v.ntot()];  
-      memcpy(f_backup[c][cmp], 
-	     f_prev[c][cmp] ? f_prev[c][cmp] : f[c][cmp],
-	     v.ntot()*sizeof(double));    
+      BACKUP(f);
+      BACKUP(f_u);
+      BACKUP(f_w);
+      BACKUP(f_cond);
+
+#undef BACKUP
     }
   }
 }
   
 void fields_chunk::restore_component(component c) {
   DOCMP if (f_backup[c][cmp]) {
-    if (f[c][cmp])
-      memcpy(f[c][cmp], f_prev[c][cmp] ? f_prev[c][cmp] : f_backup[c][cmp], 
-	     v.ntot()*sizeof(double));
-    if (f_prev[c][cmp])
-      memcpy(f_prev[c][cmp], f_backup[c][cmp], v.ntot()*sizeof(double));
+#define RESTORE(f)							\
+    if (f[c][cmp])							\
+      memcpy(f[c][cmp], f##_backup[c][cmp], gv.ntot()*sizeof(realnum));
+    
+    RESTORE(f);
+    RESTORE(f_u);
+    RESTORE(f_w);
+    RESTORE(f_cond);
+    
+#undef RESTORE
   }
 }
 
 void fields_chunk::average_with_backup(component c) {
   DOCMP {
-    double *fc = f[c][cmp];
-    double *backup = f_prev[c][cmp] ? f_prev[c][cmp] : f_backup[c][cmp];
+    realnum *fc = f[c][cmp];
+    realnum *backup = f_backup[c][cmp];
     if (fc && backup)
-      for (int i = 0; i < v.ntot(); i++)
+      for (int i = 0; i < gv.ntot(); i++)
 	fc[i] = 0.5 * (fc[i] + backup[i]);
   }
 }
@@ -149,10 +157,12 @@ void fields::synchronize_magnetic_fields() {
       FOR_MAGNETIC_COMPONENTS(c) chunks[i]->backup_component(c);
     }
   am_now_working_on(Stepping);
+  calc_sources(time()); // for B sources
   step_db(B_stuff);
-  step_b_source();
+  step_source(B_stuff);
   step_boundaries(B_stuff);
-  update_h_from_b();
+  calc_sources(time() + 0.5*dt); // for integrated H sources
+  update_eh(H_stuff);
   step_boundaries(H_stuff);
   finished_working();
   for (int i=0;i<num_chunks;i++) 
@@ -182,25 +192,25 @@ static void thermo_chunkloop(fields_chunk *fc, int ichunk, component cgrid,
 			     void *sum_) {
   long double *sum = (long double *) sum_;
   (void)shift; (void)shift_phase; (void)S; (void)sn; (void)ichunk; // unused
-  for (polarization *pol = fc->pol; pol; pol = pol->next)
+  for (polarization *pol = fc->pols[type(cgrid)]; pol; pol = pol->next)
     if (pol->energy[cgrid])
-      LOOP_OVER_IVECS(fc->v, is, ie, idx)
+      LOOP_OVER_IVECS(fc->gv, is, ie, idx)
 	*sum += IVEC_LOOP_WEIGHT(s0, s1, e0, e1, dV0 + dV1 * loop_i2)
 	* pol->energy[cgrid][idx];
 }
 
-double fields::thermo_energy_in_box(const geometric_volume &where) {
+double fields::thermo_energy_in_box(const volume &where) {
   long double sum = 0.0;
-  FOR_ELECTRIC_COMPONENTS(c)
-    if (!coordinate_mismatch(v.dim, c))
+  FOR_COMPONENTS(c)
+    if (!coordinate_mismatch(gv.dim, c))
       loop_in_chunks(thermo_chunkloop, (void *) &sum, where, c);
   return sum_to_all(sum);
 }
 
 /* Compute ExH integral in box using current fields, ignoring fact
    that this E and H correspond to different times. */
-double fields::flux_in_box_wrongH(direction d, const geometric_volume &where) {
-  if (coordinate_mismatch(v.dim, d))
+double fields::flux_in_box_wrongH(direction d, const volume &where) {
+  if (coordinate_mismatch(gv.dim, d))
     return 0.0;
 
   component cE[2], cH[2];
@@ -210,7 +220,7 @@ double fields::flux_in_box_wrongH(direction d, const geometric_volume &where) {
   case R: cE[0] = Ep, cE[1] = Ez, cH[0] = Hz, cH[1] = Hp; break;
   case P: cE[0] = Ez, cE[1] = Er, cH[0] = Hr, cH[1] = Hz; break;
   case Z:
-    if (v.dim == Dcyl)
+    if (gv.dim == Dcyl)
       cE[0] = Er, cE[1] = Ep, cH[0] = Hp, cH[1] = Hr;
     else
       cE[0] = Ex, cE[1] = Ey, cH[0] = Hy, cH[1] = Hx; 
@@ -227,32 +237,32 @@ double fields::flux_in_box_wrongH(direction d, const geometric_volume &where) {
   return sum;
 }
 
-double fields::flux_in_box(direction d, const geometric_volume &where) {
+double fields::flux_in_box(direction d, const volume &where) {
   synchronize_magnetic_fields();
   double cur_step_flux = flux_in_box_wrongH(d, where);
   restore_magnetic_fields();
   return cur_step_flux;
 }
 
-flux_vol *fields::add_flux_vol(direction d, const geometric_volume &where) {
-  if (where.dim != v.dim) abort("invalid dimensionality in add_flux_vol");
-  if (d == NO_DIRECTION || coordinate_mismatch(v.dim, d))
+flux_vol *fields::add_flux_vol(direction d, const volume &where) {
+  if (where.dim != gv.dim) abort("invalid dimensionality in add_flux_vol");
+  if (d == NO_DIRECTION || coordinate_mismatch(gv.dim, d))
     abort("invalid direction in add_flux_vol");
  return new flux_vol(this, d, where);
 }
 
 // As add_flux_vol, but infer direction from where (if possible)
-flux_vol *fields::add_flux_plane(const geometric_volume &where) {
+flux_vol *fields::add_flux_plane(const volume &where) {
   return add_flux_vol(where.normal_direction(), where);
 }
 
 flux_vol *fields::add_flux_plane(const vec &p1, const vec &p2) {
-  return add_flux_plane(geometric_volume(p1, p2));
+  return add_flux_plane(volume(p1, p2));
 }
 
 /************************************************************************/
 
-/* Note that computation of modal volume by this definition is
+/* Note that computation of modal grid_volume by this definition is
    somewhat problematic computationally, because we need to compute
    max|D*E|, which requires averaging discontinuous functions.  Hence,
    except for the special case of 2d TM polarization, the computed
@@ -260,7 +270,7 @@ flux_vol *fields::add_flux_plane(const vec &p1, const vec &p2) {
    dielectric boundary as it commonly does. 
 
    A better method would be to average only continuous quantities in
-   order to compute the fields on the Dielectric grid, but this
+   order to compute the fields on the Centered grid, but this
    is more expensive and requires us to know the boundary orientation, and
    does not seem worth the trouble at this point. */
 
@@ -273,9 +283,9 @@ static complex<double> dot3_max_integrand(const complex<double> *fields,
 	  real(conj(fields[2]) * fields[5]));
 }
 
-double fields::electric_energy_max_in_box(const geometric_volume &where) {
+double fields::electric_energy_max_in_box(const volume &where) {
   component cs[6];
-  if (v.dim == Dcyl) {
+  if (gv.dim == Dcyl) {
     cs[0] = Er; cs[1] = Ep; cs[2] = Ez;
     cs[3+0] = Dr; cs[3+1] = Dp; cs[3+2] = Dz;
   }
@@ -287,10 +297,10 @@ double fields::electric_energy_max_in_box(const geometric_volume &where) {
   return max_abs(6, cs, dot3_max_integrand, 0, where) * 0.5;
 }
 
-/* "modal" volume according to definition in:
+/* "modal" grid_volume according to definition in:
       E. M. Purcell, Phys. Rev. B 69, 681 (1946).
     (based on spontaneous emission enhancement). */
-double fields::modal_volume_in_box(const geometric_volume &where) {
+double fields::modal_volume_in_box(const volume &where) {
   return electric_energy_in_box(where) / electric_energy_max_in_box(where);
 }
 
@@ -310,10 +320,10 @@ static complex<double> dot_fx_integrand(const complex<double> *fields,
 
 /* computes integral of f(x) * |E|^2 / integral epsilon*|E|^2 */
 double fields::electric_sqr_weighted_integral(double (*f)(const vec &),
-					     const geometric_volume &where) {
+					     const volume &where) {
   double sum = 0.0;
   FOR_ELECTRIC_COMPONENTS(c) 
-    if (!coordinate_mismatch(v.dim, component_direction(c))) {
+    if (!coordinate_mismatch(gv.dim, component_direction(c))) {
       component cs[2];
       cs[0] = cs[1] = direction_component(Ex, component_direction(c));
       sum += real(integrate(2, cs, dot_fx_integrand, (void *) f, where));
@@ -323,10 +333,10 @@ double fields::electric_sqr_weighted_integral(double (*f)(const vec &),
 
 /* computes integral of f(x) * epsilon*|E|^2 / integral epsilon*|E|^2 */
 double fields::electric_energy_weighted_integral(double (*f)(const vec &),
-					     const geometric_volume &where) {
+					     const volume &where) {
   double sum = 0.0;
   FOR_ELECTRIC_COMPONENTS(c) 
-    if (!coordinate_mismatch(v.dim, component_direction(c))) {
+    if (!coordinate_mismatch(gv.dim, component_direction(c))) {
       component cs[2];
       cs[0] = direction_component(Ex, component_direction(c));
       cs[1] = direction_component(Dx, component_direction(c));
