@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2008 Massachusetts Institute of Technology  
+/* Copyright (C) 2005-2009 Massachusetts Institute of Technology  
 %
 %  This program is free software; you can redistribute it and/or modify
 %  it under the terms of the GNU General Public License as published by
@@ -23,16 +23,22 @@
 using namespace meep;
 
 double one(const vec &) { return 1.0; }
-double targets(const vec &v) {
-  const double r = sqrt(v.x()*v.x() + v.y()*v.y());
+double targets(const vec &pt) {
+  const double r = sqrt(pt.x()*pt.x() + pt.y()*pt.y());
   double dr = r;
   while (dr > 1) dr -= 1;
   if (dr > 0.7001) return 12.0;
   return 1.0;
 }
 
+#if MEEP_SINGLE
+static const double tol = 1e-3, thresh = 1e-3;
+#else
+static const double tol = 1e-9, thresh = 1e-15;
+#endif
+
 int compare(double a, double b, const char *n) {
-  if (fabs(a-b) > fabs(b)*2.0e-12) {
+  if (fabs(a-b) > fabs(b)*tol && fabs(b) > thresh) {
     master_printf("%s differs by\t%g out of\t%g\n", n, a-b, b);
     master_printf("This gives a fractional error of %g\n", fabs(a-b)/fabs(b));
     return 0;
@@ -47,9 +53,9 @@ int compare_point(fields &f1, fields &f2, const vec &p) {
   f2.get_point(&m1, p);
   for (int i=0;i<10;i++) {
     component c = (component) i;
-    if (f1.v.has_field(c)) {
+    if (f1.gv.has_field(c)) {
       complex<double> v1 = m_test.get_component(c), v2 = m1.get_component(c);
-      if (abs(v1 - v2) > 0.0*2e-15*abs(v2)) {
+      if (abs(v1 - v2) > tol*abs(v2) && abs(v2) > thresh) {
         master_printf("%s differs:  %g %g out of %g %g\n",
                component_name(c), real(v2-v1), imag(v2-v1), real(v2), imag(v2));
         master_printf("This comes out to a fractional error of %g\n",
@@ -69,9 +75,9 @@ int approx_point(fields &f1, fields &f2, const vec &p) {
   f2.get_point(&m1, p);
   for (int i=0;i<10;i++) {
     component c = (component) i;
-    if (f1.v.has_field(c)) {
+    if (f1.gv.has_field(c)) {
       complex<double> v1 = m_test.get_component(c), v2 = m1.get_component(c);
-      if (abs(v1 - v2) > 1e-14*(5.0 + abs(v2))) {
+      if (abs(v1 - v2) > tol*abs(v2) && abs(v2) > thresh) {
         master_printf("%s differs:  %g %g out of %g %g\n",
                component_name(c), real(v2-v1), imag(v2-v1), real(v2), imag(v2));
         master_printf("This comes out to a fractional error of %g\n",
@@ -89,9 +95,9 @@ int test_metal(double eps(const vec &), int splitting, const char *mydirname) {
   double a = 10.0;
   double ttot = 17.0;
 
-  volume v = vol3d(1.5, 0.5, 1.0, a);
-  structure s1(v, eps);
-  structure s(v, eps, no_pml(), identity(), splitting);
+  grid_volume gv = vol3d(1.5, 0.5, 1.0, a);
+  structure s1(gv, eps);
+  structure s(gv, eps, no_pml(), identity(), splitting);
   s.set_output_directory(mydirname);
   s1.set_output_directory(mydirname);
 
@@ -110,11 +116,11 @@ int test_metal(double eps(const vec &), int splitting, const char *mydirname) {
     if (f.time() >= total_energy_check_time) {
       if (!compare(f.total_energy(), f1.total_energy(),
                    "   total energy")) return 0;
-      if (!compare(f.electric_energy_in_box(v.surroundings()),
-                   f1.electric_energy_in_box(v.surroundings()),
+      if (!compare(f.electric_energy_in_box(gv.surroundings()),
+                   f1.electric_energy_in_box(gv.surroundings()),
                    "electric energy")) return 0;
-      if (!compare(f.magnetic_energy_in_box(v.surroundings()),
-                   f1.magnetic_energy_in_box(v.surroundings()),
+      if (!compare(f.magnetic_energy_in_box(gv.surroundings()),
+                   f1.magnetic_energy_in_box(gv.surroundings()),
                    "magnetic energy")) return 0;
       total_energy_check_time += 5.0;
     }
@@ -126,9 +132,9 @@ int test_periodic(double eps(const vec &), int splitting, const char *mydirname)
   double a = 10.0;
   double ttot = 17.0;
 
-  volume v = vol3d(1.5, 0.5, 1.0, a);
-  structure s1(v, eps);
-  structure s(v, eps, no_pml(), identity(), splitting);
+  grid_volume gv = vol3d(1.5, 0.5, 1.0, a);
+  structure s1(gv, eps);
+  structure s(gv, eps, no_pml(), identity(), splitting);
   s.set_output_directory(mydirname);
   s1.set_output_directory(mydirname);
 
@@ -149,11 +155,11 @@ int test_periodic(double eps(const vec &), int splitting, const char *mydirname)
     if (f.time() >= total_energy_check_time) {
       if (!compare(f.total_energy(), f1.total_energy(),
                    "   total energy")) return 0;
-      if (!compare(f.electric_energy_in_box(v.surroundings()),
-                   f1.electric_energy_in_box(v.surroundings()),
+      if (!compare(f.electric_energy_in_box(gv.surroundings()),
+                   f1.electric_energy_in_box(gv.surroundings()),
                    "electric energy")) return 0;
-      if (!compare(f.magnetic_energy_in_box(v.surroundings()),
-                   f1.magnetic_energy_in_box(v.surroundings()),
+      if (!compare(f.magnetic_energy_in_box(gv.surroundings()),
+                   f1.magnetic_energy_in_box(gv.surroundings()),
                    "magnetic energy")) return 0;
       total_energy_check_time += 5.0;
     }
@@ -164,8 +170,8 @@ int test_periodic(double eps(const vec &), int splitting, const char *mydirname)
 int test_pml(double eps(const vec &), const char *mydirname) {
   double a = 10.0;
 
-  volume v = vol3d(1.5, 1.0, 1.2, a);
-  structure s(v, eps, pml(0.401));
+  grid_volume gv = vol3d(1.5, 1.0, 1.2, a);
+  structure s(gv, eps, pml(0.401));
   s.set_output_directory(mydirname);
 
   master_printf("Testing pml quality...\n");
@@ -182,12 +188,11 @@ int test_pml(double eps(const vec &), const char *mydirname) {
     f.step();
     if (f.time() >= total_energy_check_time) {
       const double new_energy = f.total_energy();
-      if (new_energy > last_energy*4e-3) { // FIXME: problem here? this is pretty slow...
+      master_printf("Got newE/oldE of %g\n", new_energy/last_energy);
+      if (new_energy > last_energy*4e-3) {
         master_printf("Energy decaying too slowly: from %g to %g (%g)\n",
                       last_energy, new_energy, new_energy/last_energy);
         return 0;
-      } else {
-        master_printf("Got newE/oldE of %g\n", new_energy/last_energy);
       }
       total_energy_check_time += deltaT;
     }
@@ -198,9 +203,9 @@ int test_pml(double eps(const vec &), const char *mydirname) {
 int test_pml_splitting(double eps(const vec &), int splitting, const char *mydirname) {
   double a = 10.0;
 
-  volume v = vol3d(1.5, 1.0, 1.2, a);
-  structure s1(v, eps, pml(0.3));
-  structure s(v, eps, pml(0.3), identity(), splitting);
+  grid_volume gv = vol3d(1.5, 1.0, 1.2, a);
+  structure s1(gv, eps, pml(0.3));
+  structure s(gv, eps, pml(0.3), identity(), splitting);
   s.set_output_directory(mydirname);
   s1.set_output_directory(mydirname);
 
